/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */
/*
 * Copyright (c) Qualcomm Technologies, Inc. and/or its subsidiaries.
 */

#ifndef _DT_BINDINGS_CLK_QCOM_TCSR_CC_HAWI_H
#define _DT_BINDINGS_CLK_QCOM_TCSR_CC_HAWI_H

/* TCSR_CC clocks */
#define TCSR_PCIE_0_CLKREF_EN					0
#define TCSR_PCIE_1_CLKREF_EN					1
#define TCSR_UFS_CLKREF_EN					2
#define TCSR_USB2_CLKREF_EN					3
#define TCSR_USB3_CLKREF_EN					4

#endif
