/* SPDX-License-Identifier: GPL-2.0 */
#ifndef __LINUX_DMA_XILINX_DPDMA_H
#define __LINUX_DMA_XILINX_DPDMA_H

#include <linux/types.h>

struct xilinx_dpdma_peripheral_config {
	bool video_group;
};

#endif /* __LINUX_DMA_XILINX_DPDMA_H */
