/* SPDX-License-Identifier: GPL-2.0-only */
/* Copyright (c) 2013, The Linux Foundation. All rights reserved.
 */
#ifndef __DT_BINDINGS_QCOM_GSBI_H
#define __DT_BINDINGS_QCOM_GSBI_H

#define GSBI_PROT_IDLE		0
#define GSBI_PROT_I2C_UIM	1
#define GSBI_PROT_I2C		2
#define GSBI_PROT_SPI		3
#define GSBI_PROT_UART_W_FC	4
#define GSBI_PROT_UIM		5
#define GSBI_PROT_I2C_UART	6

#define GSBI_CRCI_QUP		0
#define GSBI_CRCI_UART		1

#endif
