/* SPDX-License-Identifier: GPL-2.0 */
#ifndef __STE_CLK_DB8500_CLKOUT_H__
#define __STE_CLK_DB8500_CLKOUT_H__

#define DB8500_CLKOUT_1			0
#define DB8500_CLKOUT_2			1

#define DB8500_CLKOUT_SRC_CLK38M	0
#define DB8500_CLKOUT_SRC_ACLK		1
#define DB8500_CLKOUT_SRC_SYSCLK	2
#define DB8500_CLKOUT_SRC_LCDCLK	3
#define DB8500_CLKOUT_SRC_SDMMCCLK	4
#define DB8500_CLKOUT_SRC_TVCLK		5
#define DB8500_CLKOUT_SRC_TIMCLK	6
#define DB8500_CLKOUT_SRC_CLK009	7

#endif
